@@ -39,8 +39,10 @@ main(int argc, char *argv[])
 			seedrandom((unsigned char *) argv[i], k);
 			zap((unsigned char *) argv[i], k);
 		}
+#ifndef BATCH
 	fprintf(stderr, "please enter a random string -- end with <CR>\n");
 	(void) getstring(0);
+#endif
 	for (i = 0; i < LKEYLEN; i++)
 		key[i] = randombyte();
 
