@@ -23,7 +23,7 @@ char rbuf[2048];
     printf("%s = %s\n", label,rbuf); }
 #endif
 
-unsigned char ver[] = "@(#)L3 1.5b - Mark C. Henderson - markh@wimsey.bc.ca";
+unsigned char ver[] = "@(#)L3 1.7.5 - Mark C. Henderson - markh@wimsey.bc.ca";
 
 #ifndef STAND
 extern void ferr(char *);
@@ -1226,9 +1226,9 @@ void halfluc(MP_INT * s, MP_INT * m, MP_INT * e, MP_INT * n,
         24  3 IVs
 really, I shouldn't bother encrypting the IVs
 */
-/* LKEYLEN must match value defined in l3.h */
-#define LKEYLEN 80
-#define HASHLEN 16
+/* LKEYLEN and HASHLEN must match value defined in l3.h */
+#define LKEYLEN 96
+#define HASHLEN 32
 
 static INLINE int bytes(MP_INT * n)
 {
@@ -1386,7 +1386,7 @@ int vsign(MP_INT * c, unsigned char *hash, MP_INT * n, MP_INT * e, int alg)
 	else
 		ferr("vsign: invalid paramaters\n");
 	mptcs(ribuf, nn, m);
-	for (i = 0; i < 16; i++)
+	for (i = 0; i < HASHLEN; i++)
 		if (hash[i] != ribuf[nn - HASHLEN + i])
 			return 0;
 	for (i = 1; i < nn - HASHLEN; i++)
