@@ -9,7 +9,8 @@
 #include <utime.h>
 #include "l3.h"
 
-unsigned char key[56];
+unsigned char key[72];
+unsigned char k2buf[32];
 unsigned char iv[24];
 unsigned char eiv[24];
 extern unsigned char ver[];
@@ -300,6 +301,7 @@ char **argv;
 	}
 	zap(eiv, sizeof(eiv));
 	zap(key, sizeof(key));
+	zap(k2buf, sizeof(k2buf));
 	zap(iv, sizeof(iv));
 	closerandom();
 	exit(perm_stat ? perm_stat : exit_stat);
@@ -335,39 +337,24 @@ int decrypt;
 		zap(tt, strlen(tt));
 		mdstring(ss, strlen(ss), key);
 		sn512string(ss, strlen(ss), &key[16]);
-		if (debug) {
-			fprintf(stderr, "\npto key:\n   ");
-			for (i = 0; i < 24; i++)
-				fprintf(stderr, "%02x", key[i]);
-			fprintf(stderr, "\n   ");
-			for (i = 24; i < 48; i++)
-				fprintf(stderr, "%02x", key[i]);
-			fprintf(stderr, "\n");
-		}
-
-		if (debug) {
-			fprintf(stderr, "\nraw key:\n   ");
-			for (i = 0; i < 24; i++)
-				fprintf(stderr, "%02x", key[i]);
-			fprintf(stderr, "\n   ");
-			for (i = 24; i < 48; i++)
-				fprintf(stderr, "%02x", key[i]);
-			fprintf(stderr, "\n");
-		}
+		haval_string(ss, k2buf);
 		zap(ss, strlen(tt));
+		for (i=0; i<24; i++)
+			key[48+i] = k2buf[i];
+		zap(k2buf,sizeof(k2buf));
+		
 	}
 	havekey = 1;
-	/* expand key - from 48 to 56 bytes */
-	des3key(&key[2], 1);
-	Ddes(&key[34], &key[48]);
-	key[48] ^= key[0] ^ key[3];
-	key[49] ^= key[12] ^ key[45];
-	key[50] ^= key[31] ^ key[23];
-	key[51] ^= key[25] ^ key[17];
-	key[52] ^= key[3] ^ key[12];
-	key[53] ^= key[46] ^ key[27];
-	key[54] ^= key[24] ^ key[14];
-	key[55] ^= key[7] ^ key[37];
+
+	if (debug) {
+		fprintf(stderr, "\nraw key:\n   ");
+		for (i = 0; i < 36; i++)
+			fprintf(stderr, "%02x", key[i]);
+		fprintf(stderr, "\n   ");
+		for (i = 36; i < 72; i++)
+			fprintf(stderr, "%02x", key[i]);
+		fprintf(stderr, "\n");
+	}
 	if (!decrypt) {
 		if (!test_mode)
 			for (i = 0; i < 24; i++)
@@ -464,6 +451,8 @@ char *ifname, *ofname;
 #endif
 
 	fclose(stdout);
+	if (debug)
+		fprintf(stderr, "copystat(%s,%s)\n",ifname,ofname);
 	if (validate && !do_decomp && !mcat_flg) {
 		struct stat qq;
 		FILE *chkfile;
@@ -476,21 +465,12 @@ char *ifname, *ofname;
 			fprintf(stderr, "cannot stat %s\n", infile);
 			exit(1);
 		}
-		/* expand key - just a bit */
-		des3key(&key[2], 1);
-		Ddes(&key[34], &key[48]);
-		key[48] ^= key[0] ^ key[3];
-		key[49] ^= key[12] ^ key[45];
-		key[50] ^= key[31] ^ key[23];
-		key[51] ^= key[25] ^ key[17];
-		key[52] ^= key[3] ^ key[12];
-		key[53] ^= key[46] ^ key[27];
-		key[54] ^= key[24] ^ key[14];
-		key[55] ^= key[7] ^ key[37];
 		ret = scramble(stdin, stdout, key, iv, 1);
-		if (ret == 0 && !quiet)
+	    if (debug)
+		   fprintf(stderr, "validation scramble returns %d\n", ret);
+		if (ret != 0 && !quiet)
 			fprintf(stderr, "validation of %s successful\n", ofname);
-		if (ret < 0) {
+		if (ret == 0) {
 			fprintf(stderr, "validation of %s failed\n", ofname);
 			fprintf(stderr, "warning: neither %s nor %s will be removed\n", ifname, ofname);
 			exit_stat = 1;
@@ -569,6 +549,7 @@ oops()
 {
 	zap(eiv, sizeof(eiv));
 	zap(key, sizeof(key));
+	zap(k2buf, sizeof(k2buf));
 	zap(iv, sizeof(iv));
 	if (do_decomp)
 		fprintf(stderr, "corrupt input\n");
