@@ -1,4 +1,3 @@
-
 /* misc.c - Mark Henderson (markh@wimsey.bc.ca) */
 #include "l3.h"
 void ferr(char *s)
@@ -91,14 +90,14 @@ int read_number(FILE * f, MP_INT * x)
 zap(mbuf,8); zap(nbuf,8), zap(buf,8); zap(sbuf,8); zap(bigbuf,4096); \
 zap(iv,24)
 
-/* speaking of amateur theatrics... This is slow. */
 int scramble(FILE * in, FILE * out, unsigned char *key, unsigned char *iiv,
 			 int decrypt)
-/* key is 56 bytes, iv is 24 bytes */
+/* key is 72 bytes, iv is 24 bytes , 72 + 24 == 96 == LKEYLEN */
 {
 	unsigned char buf[8], nbuf[8];
 	unsigned char mbuf[8], sbuf[8];
 	unsigned char iv[24];
+	BFkey_type     *bfkey ;
 	unsigned char bigbuf[4096];
 	unsigned char *t, *s;
 	int count;
@@ -107,11 +106,26 @@ int scramble(FILE * in, FILE * out, unsigned char *key, unsigned char *iiv,
 	int ret_val = 1;
 	clear_tran();
 
+#ifdef DEBUG
+	fprintf(stderr, "scramble key\n ");
+	for (i=0; i<36; i++)
+		fprintf(stderr, "%02x", key[i]);
+	fprintf(stderr, "\n ");
+	for (i=36; i<72; i++)
+		fprintf(stderr, "%02x", key[i]);
+	fprintf(stderr, "\n");
+#endif
+
 	des4key(&key[0], decrypt);	/* bytes 0-31 */
 	initkey_idea(&key[32], decrypt);	/* bytes 32-47 */
 	/* bytes 48 - 55 are used for tran */
 	memcpy(iv, iiv, 24);  /* don't damage input iv */
 
+	if (!(bfkey = make_bfkey(&key[56], (short)16))) {
+		fprintf(stderr, "fatal key setup error\n");
+		exit(1);
+	}
+
 	if (!decrypt) {	/* encrypt */
 		must_pad = 1;	/* assume the worst */
 		while (((i = fread(&bigbuf[0], 1, 4096, in)) > 0) || must_pad) {
@@ -136,6 +150,7 @@ int scramble(FILE * in, FILE * out, unsigned char *key, unsigned char *iiv,
 				for (t = &bigbuf[j], s = mbuf, count = 8; count; count--)
 					*(t++) ^= *(s++);
 				idea_ecb((word16 *) & bigbuf[j], (word16 *) & bigbuf[j]);
+				crypt_8bytes(&bigbuf[j],&bigbuf[j],bfkey,(short)0);
 				memcpy(mbuf, &bigbuf[j], 8);
 			}
 			memcpy(&iv[8], mbuf, 8);
@@ -146,6 +161,7 @@ int scramble(FILE * in, FILE * out, unsigned char *key, unsigned char *iiv,
 					*(t++) ^= *(s++);
 				des4(&bigbuf[j], &bigbuf[j]);
 				idea_ecb((word16 *) & bigbuf[j], (word16 *) & bigbuf[j]);
+				crypt_8bytes(&bigbuf[j],&bigbuf[j],bfkey,(short)0);
 				Ddes(&bigbuf[j], &bigbuf[j]);
 				memcpy(mbuf, &bigbuf[j], 8);
 			}
@@ -165,6 +181,7 @@ int scramble(FILE * in, FILE * out, unsigned char *key, unsigned char *iiv,
 				memcpy(nbuf, &bigbuf[j], 8);
 				memcpy(sbuf, buf, 8);
 				Ddes(buf,buf);
+				crypt_8bytes(buf,buf,bfkey,(short)1);
 				idea_ecb((word16 *)buf,(word16 *)buf);
 				des4(buf, buf);
 				for (t = buf, s = mbuf, count = 8; count; count--)
@@ -175,6 +192,7 @@ int scramble(FILE * in, FILE * out, unsigned char *key, unsigned char *iiv,
 			}
 			memcpy(sbuf, buf, 8);
 			Ddes(buf,buf);
+			crypt_8bytes(buf,buf,bfkey,(short)1);
 			idea_ecb((word16 *)buf,(word16 *)buf);
 			des4(buf, buf);
 			for (t = buf, s = mbuf, count = 8; count; count--)
@@ -187,6 +205,7 @@ int scramble(FILE * in, FILE * out, unsigned char *key, unsigned char *iiv,
 			for (j = 8; j < i; j += 8) {
 				memcpy(nbuf, &bigbuf[j], 8);
 				memcpy(sbuf, buf, 8);
+				crypt_8bytes(buf,buf,bfkey,(short)1);
 				idea_ecb((word16 *) buf, (word16 *) buf);
 				for (t = buf, s = mbuf, count = 8; count; count--)
 					*(t++) ^= *(s++);
@@ -195,6 +214,7 @@ int scramble(FILE * in, FILE * out, unsigned char *key, unsigned char *iiv,
 				memcpy(buf, nbuf, 8);
 			}
 			memcpy(sbuf, buf, 8);
+			crypt_8bytes(buf,buf,bfkey,(short)1);
 			idea_ecb((word16 *) buf, (word16 *) buf);
 			for (t = buf, s = mbuf, count = 8; count; count--)
 				*(t++) ^= *(s++);
@@ -241,6 +261,7 @@ int scramble(FILE * in, FILE * out, unsigned char *key, unsigned char *iiv,
 	close_idea();
 	close_des();
 	clear_tran();
+	destroy_bfkey(bfkey);
 	cleanup;
 	return ret_val;
 }
@@ -267,6 +288,30 @@ void mdstring(char *s, unsigned int len, unsigned char *digest)
 	memcpy(digest, &mdContext.digest[0], 16);
 }
 
+/* this really shouldn't depend on MAXC, but it does -should fix, not hard */
+void havalpkid(MP_INT *n, MP_INT *e, unsigned char *id, unsigned char *digest)
+{
+	unsigned char buf[MAXC];
+	haval_state state;
+#ifdef TEST
+	int i;
+#endif
+	haval_start(&state);
+	mptcs(buf, MAXC, n);
+	haval_hash(&state,&buf[0],(unsigned int)MAXC);
+	mptcs(buf, MAXC, e);
+	haval_hash(&state,&buf[0],(unsigned int)MAXC);
+	haval_hash(&state,id,(unsigned int)strlen(id));
+	haval_end(&state,digest);
+#ifdef TEST
+	fprintf(stderr, "havalpkid returns digest:\n  ");
+	for (i=0; i<HASHLEN; i++)
+		fprintf(stderr, "%02x",digest[i]);
+	fprintf(stderr, "\n");
+#endif
+}
+
+/* this really shouldn't depend on MAXC, but it does -should fix, not hard */
 void mdpkid(MP_INT * n, MP_INT * e, unsigned char *id, unsigned char *digest)
 {
 	unsigned char buf[MAXC];
@@ -485,7 +530,7 @@ int readpk(FILE * f, MP_INT * n, MP_INT * e)
 {
 	int salg;
 	unsigned char idstring[128];
-	char hash[16];
+	char hash[HASHLEN];
 	MP_INT *c;
 
 	if (!read_number(f, n) || !read_number(f, e)) {
@@ -498,7 +543,7 @@ int readpk(FILE * f, MP_INT * n, MP_INT * e)
 	}
 	else {
 		init(c);
-		mdpkid(n, e, idstring, hash);
+		havalpkid(n, e, idstring, hash);
 		if (!readalg(f, &salg) || !read_number(f, c) ||
 			!vsign(c, hash, n, e, salg)) {
 			fprintf(stderr,
