@@ -9,7 +9,7 @@ main(int argc, char *argv[])
 	int i;
 	unsigned char k[16];
 	unsigned char iv[8];
-	unsigned char hash[16];
+	unsigned char hash[HASHLEN];
 	unsigned char *ss;
 	FILE *private;
 	init(e);
@@ -40,14 +40,14 @@ main(int argc, char *argv[])
 	if (!read_numberx(private, p, k, iv) || !read_numberx(private, q, k, iv)) {
 		fprintf(stderr, "wrong key to key entered or key corrupt\n");
 		zap(k, 16);
-		zap(hash, 16);
+		zap(hash, sizeof(hash));
 		zap(ss, strlen(ss));
 		exit(1);
 	}
 
 	read_number(private, e);
 
-	mdfile(stdin, hash);
+	haval_file_2(stdin,hash);
 	sign(c, hash, p, q, e, alg);
 	fputc((char) alg, stdout);
 	write_number(stdout, c);
