@@ -24,7 +24,7 @@ main(int argc, char *argv[])
 	unsigned char k[16];
 	unsigned char iv[8];
 	unsigned char idstring[127];
-	unsigned char hash[16];
+	unsigned char hash[HASHLEN];
 	int alg = getalg();
 	unsigned char *ss;
 	FILE *private;
@@ -71,7 +71,7 @@ main(int argc, char *argv[])
 	if (!(public = fopen(pubfile, "wb"))) {
 		fprintf(stderr, "could not open %s for write\n", pubfile);
 		zap(k, 16);
-		zap(hash, 16);
+		zap(hash, HASHLEN);
 		zap(ss, strlen(ss));
 		exit(1);
 	}
@@ -80,7 +80,7 @@ main(int argc, char *argv[])
 	if (!fgets(idstring, 127, stdin)) {
 		fprintf(stderr, "could not read id from stdin\n");
 		zap(k, 16);
-		zap(hash, 16);
+		zap(hash, HASHLEN);
 		zap(ss, strlen(ss));
 		exit(1);
 	}
@@ -89,11 +89,11 @@ main(int argc, char *argv[])
 		idstring[idlen - 1] = '\0';
 		idlen--;
 	}
-	mdpkid(n, e, idstring, hash);
+	havalpkid(n, e, idstring, hash);
 
 #ifdef DEBUG
 	fprintf(stderr, "hash of (%s,n,e) = ", idstring);
-	for (i = 0; i < 16; i++)
+	for (i = 0; i < HASHLEN; i++)
 		fprintf(stderr, "%02x", hash[i]);
 	fprintf(stderr, "\n");
 #endif
@@ -106,6 +106,6 @@ main(int argc, char *argv[])
 	write_number(public, c);
 	fclose(public);
 	zap(k, 16);
-	zap(hash, 16);
+	zap(hash, sizeof(hash));
 	zap(ss, strlen(ss));
 }
