@@ -7,7 +7,7 @@ main(int argc, char *argv[])
 	char pubfile[128];
 	int i;
 	int alg = getalg();
-	unsigned char hash[16];
+	unsigned char hash[HASHLEN];
 	unsigned char sig[MAXC];
 	FILE *public;
 	FILE *sigfile;
@@ -44,7 +44,7 @@ main(int argc, char *argv[])
 		exit(1);
 	}
 
-	mdfile(stdin, hash);
+	haval_file_2(stdin, hash);
 	if (vsign(c, hash, n, e, alg)) {
 		printf("file signature verified\n");
 	}
@@ -52,5 +52,6 @@ main(int argc, char *argv[])
 		printf("file signature did not verify\n");
 		exit(1);
 	}
+	zap(hash,sizeof(hash));
 	exit(0);
 }
